﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

[System.Serializable]
public class Effect
{
    public string Name;
    public AudioClip Clip;
}

public class AudioPlayer : MonoBehaviour {
    public bool DisableInEditor = true;

    public AudioSource[] Sources;
    static AudioSource[] sources;

    public List<Effect> Effects;
    static Dictionary<string, List<AudioClip>> effects;

    static int lastChannel = 0;

	void Awake()
	{
        effects = new Dictionary<string, List<AudioClip>>();
        foreach(Effect e in Effects)
        {
            if(!effects.ContainsKey(e.Name))
            {
                effects[e.Name] = new List<AudioClip>();
            }
            effects[e.Name].Add(e.Clip);
        }

        sources = Sources;

#if UNITY_EDITOR
        if(DisableInEditor)
        {
            gameObject.SetActive(false);
        }
#endif
	}
	
	public static void PlayEffect(string name)
    {
        if(effects.ContainsKey(name))
        {
            sources[lastChannel++].PlayOneShot(effects[name][Random.Range(0, effects[name].Count)]);
            if(lastChannel >= sources.Length)
            {
                lastChannel = 0;
            }
        }
        else
        {
            Debug.Log("No effect with name " + name + " found!");
        }
    }
}
