﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public class Breakable : MonoBehaviour {
    public GameObject Explosion;

    public bool NoiseOnBreak;
    public float NoiseRadius;
    public float Spookiness;
    public string AudioEffectName;

    public float BreakForce;

    Transform thisTransform;

    public LayerMask BreakLayer;

    void Awake()
    {
        thisTransform = transform;
    }

    void OnCollisionEnter2D(Collision2D c)
    {
        if (((BreakLayer.value & (1 << c.gameObject.layer)) > 0) || (c.rigidbody != null && c.relativeVelocity.magnitude * c.rigidbody.mass > BreakForce))
        {
            Break(c.relativeVelocity);

            Person p = c.gameObject.GetComponent<Person>();
            if(p != null)
            {
                p.Kill();
            }
        }

    }

    public virtual void Break(Vector3 direction)
    {
        Collider2D[] colliders = Physics2D.OverlapCircleAll(thisTransform.position, NoiseRadius);

        foreach(Collider2D col in colliders)
        {
            Person p = col.GetComponent<Person>();

            if(p != null)
            {
                p.Scare(Spookiness);
            }
        }

        if (Explosion != null)
        {
            GameObject g = Instantiate(Explosion, thisTransform.position, Quaternion.identity) as GameObject;
            float dir = direction.x > 0 ? 0 : 1;
            g.transform.Rotate(0,dir * 180f,0);
        }

        if(string.IsNullOrEmpty(AudioEffectName))
        {
            AudioPlayer.PlayEffect(AudioEffectName);
        }

        Destroy(gameObject);
    }
}
