﻿using UnityEngine;
using System.Collections;

public enum PersonState
{
    Idle,
    Spooked,
    Scared,
    Terrified,
    Dead
}

public class Person : MonoBehaviour {
    public float SpeedIncrease;

    public float ScareAmount;
    public float ScareState1Amount;
    public float ScareState2Amount;
    public float ScareState3Amount;
    public float ScareLimit;
    public PersonState State;

	public float Velocity;
    public float currVelocity;
	Rigidbody2D body;
	
	Vector3 velocity;
    float distanceToGround;

	Animator thisAnimator;
	Transform thisTransform;
    Collider2D thisCollider;

	float dir = 0.0f;
    float nextDirectionUpdate;
    Vector3 lastPos;

    public LayerMask groundLayer;

    public bool Grounded;

	void Awake () {
		body = GetComponent<Rigidbody2D> ();
		thisAnimator = GetComponentInChildren<Animator>();
		thisTransform = GetComponent<Transform> ();
        thisCollider = GetComponent<Collider2D>();

        currVelocity = Velocity;
        distanceToGround = thisCollider.bounds.extents.y;
	}
	

	void Update () {
        //Debug.Log(body.velocity);
        if (State == PersonState.Dead)
            return;

        Grounded = IsGrounded();

        if(nextDirectionUpdate < Time.time)
        {
            NewDirection();
        }

        UpdateState();
        DoState();

        if (ScareAmount > 0)
            ScareAmount -= (Time.deltaTime / 15f);
        else
            ScareAmount = 0;

        if(thisAnimator != null)
        {
            if(body.velocity.x != 0)
            {
                thisAnimator.SetBool("Walking", true);
            }
            else
            {
                thisAnimator.SetBool("Walking", false);
            }
        }
	}
	
	void FixedUpdate()
	{
        if (State == PersonState.Dead)
            return; 
        
        if (!IsGrounded())
        {
            if (body.fixedAngle)
                body.fixedAngle = false;
            return;
        } else
        {
            if (!body.fixedAngle)
                body.fixedAngle = true;
        }

		Vector3 vel = body.velocity;
		vel.x = dir * currVelocity;
		body.velocity = vel;

        lastPos = thisTransform.position;
	}

    public float Scare(float amt)
    {
        ScareAmount += amt;

        return ScareAmount;
    }

    void UpdateState()
    {
        PersonState startState = State;

        //goin' up
        if (ScareAmount > ScareState1Amount && State == PersonState.Idle)
        {
            currVelocity += SpeedIncrease;
            State = PersonState.Spooked;

            AudioPlayer.PlayEffect("Spooked");

            NewDirection();
        }
        else if (ScareAmount > ScareState2Amount && State == PersonState.Spooked)
        {
            currVelocity += SpeedIncrease;
            State = PersonState.Scared;
            AudioPlayer.PlayEffect("Scared");
        }
        else if (ScareAmount > ScareState3Amount && State == PersonState.Scared)
        {
            currVelocity += SpeedIncrease;
            State = PersonState.Terrified;
            AudioPlayer.PlayEffect("Scared");
        }
        else if (ScareAmount > ScareLimit && State == PersonState.Terrified)
        {
            Debug.Log("Scared to death.");
            //State = PersonState.Dead;
        }

        //goin' down
        if (ScareAmount < ScareState1Amount && State == PersonState.Spooked)
        {
            currVelocity -= SpeedIncrease;
            State = PersonState.Idle;
        }
        else if (ScareAmount < ScareState2Amount && State == PersonState.Scared)
        {
            currVelocity -= SpeedIncrease;
            State = PersonState.Spooked;
        }
        else if (ScareAmount < ScareState3Amount && State == PersonState.Terrified)
        {
            currVelocity -= SpeedIncrease;
            State = PersonState.Scared;
        }

        if(startState != State)
        {
            AlertNewState();
        }
    }

    void AlertNewState()
    {
        Debug.Log(gameObject.name + " is now " + State);
        FloatyTextManager.MakeText(thisTransform.position + new Vector3(0,2,0), State.ToString());
    }

    void DoState()
    {
        switch(State)
        {
            case PersonState.Spooked:
                break;
            case PersonState.Scared:
                break;
            case PersonState.Terrified:
                break;
            case PersonState.Dead:
                break;
            case PersonState.Idle:
            default:
                break;
        }
    }

    void NewDirection()
    {
        if(State == PersonState.Idle)
        {
            dir = Random.Range(-1, 2);
        }
        else
        {
            dir = (Random.Range(0, 2) * 2) - 1;
        }

        nextDirectionUpdate = Time.time + Random.Range(2f, 7f);
    }

    bool IsGrounded()
    {
        //Collider2D c = Physics2D.Raycast(thisTransform.position, -Vector3.up, distanceToGround, groundLayer).collider;
        //return (c != null && c.gameObject != null);

        return thisCollider.IsTouchingLayers(groundLayer);
    }

    Vector3 storeVel;

    public void Kill()
    {
        State = PersonState.Dead;
        body.velocity = new Vector2(dir * currVelocity, 0);
        body.AddTorque(dir * currVelocity);
        body.fixedAngle = false;
        Destroy(thisAnimator);
        Destroy(this);
    }

    public void AttachToTransform(Transform t)
    {
        body.isKinematic = true;
        body.velocity = new Vector3(0, 0, 0);
        thisCollider.enabled = false;

        thisTransform.SetParent(t);
        thisTransform.position = t.position + new Vector3(0f,2.5f,0f);

        State = PersonState.Dead;

        if (thisAnimator != null)
        {
            thisAnimator.SetBool("Walking", false);
        }
    }

    public void Detach(Vector3 inheritVelocity)
    {
        body.isKinematic = false;
        thisCollider.enabled = true;
        body.fixedAngle = false;

        thisTransform.SetParent(thisTransform.parent.parent);

        body.angularVelocity = inheritVelocity.x * 3f;
        body.velocity = inheritVelocity + new Vector3(0,5f,0);

        foreach (RotateWithVelocity r in GetComponentsInChildren<RotateWithVelocity>())
        {
            r.enabled = false;
        }
    }

    public void Kill(Vector2 force)
    {
        body.isKinematic = false;
        thisCollider.enabled = true;
        body.fixedAngle = false;

        State = PersonState.Dead;

        body.angularVelocity = force.x * 3f;
        body.velocity = force;

        foreach(RotateWithVelocity r in GetComponentsInChildren<RotateWithVelocity>())
        {
            r.enabled = false;
        }

        if (thisAnimator != null)
        {
            thisAnimator.SetBool("Walking", false);
        }
    }
}
